@@ -473,10 +473,13 @@ read_varian_2dspectrum_raw=function(directory, zero_filling=T, apodization=T){
     reticulate::py_config()
   }
   
-  env=new.env()
-  reticulate::source_python("read_varian_2dspec_raw.py",  envir = env)
   
-  spec=env$read_varian_spec2d_raw(directory, fid_file, procpar_file, zero_filling=zero_filling, apodization=apodization)
+  varian_2draw_py <- system.file("read_varian_2dspec_raw.py", package="specmine")
+  env <- new.env()
+  reticulate::source_python(varian_2draw_py,  envir = env)
+  
+
+  spec <- env$read_varian_spec2d_raw(directory, fid_file, procpar_file, zero_filling=zero_filling, apodization=apodization)
   names(spec) = c("ppms","intensities")
   names(spec$ppms) = c("F2","F1")
   return(spec)
@@ -694,10 +697,10 @@ peaklist <- function(spectrum, threshold = NULL, noise=0) {
     points <- spectrum[result]
     
     #Associate its point to its row and column
-    locations <- as.matrix(sapply(points,function(x)(which(x==spectrum,arr.ind = T))))
+    locations <- as.matrix(lapply(points,function(x)(which(x==spectrum,arr.ind = T))))
     
     #Return the data frame with peak information
-    peaks <- data.frame(rows = sapply(locations[1,],function(x)rownames(spectrum)[x]),cols = sapply(locations[2,],function(x)(colnames(spectrum)[x])))
+    peaks <- data.frame(rows = unlist(lapply(locations,function(x)rownames(spectrum)[x[1,][1]])),cols = unlist(lapply(locations,function(x)(colnames(spectrum)[x[1,][2]]))))
     
   } else {
     #No threshold given so, mean of intensity values used
